@@ -19,7 +19,7 @@
 #       GD::Graph::pie
 #       GD::Graph::mixed
 #
-# $Id: Graph.pm,v 1.49 2003/03/06 23:10:25 mgjv Exp $
+# $Id: Graph.pm,v 1.51 2003/06/16 01:25:29 mgjv Exp $
 #
 #==========================================================================
 
@@ -31,8 +31,8 @@
 
 package GD::Graph;
 
-($GD::Graph::prog_version) = '$Revision: 1.49 $' =~ /\s([\d.]+)/;
-$GD::Graph::VERSION = '1.40';
+($GD::Graph::prog_version) = '$Revision: 1.51 $' =~ /\s([\d.]+)/;
+$GD::Graph::VERSION = '1.41';
 
 use strict;
 use GD;
@@ -382,20 +382,27 @@ sub set_clr # GD::Image, r, g, b
 {
     my $self = shift; 
     return unless @_;
-    my $i;
+    my $gd = $self->{graph};
+
+    # All of this could potentially be done by using colorResolve
+    # The problem is that colorResolve doesn't return an error
+    # condition (-1) if it can't allocate a color. Instead it always
+    # returns 0.
 
     # Check if this colour already exists on the canvas
-    if (($i = $self->{graph}->colorExact(@_)) < 0) 
+    my $i = $gd->colorExact(@_);
+    # if not, allocate a new one, and return its index
+    $i = $gd->colorAllocate(@_) if $i < 0;
+    # if this fails, we should use colorClosest.
+    $i = $gd->colorClosest(@_)  if $i < 0;
+
+    # TODO Deal with antialiasing here?
+    if (0 && $self->can("setAntiAliased"))
     {
-        # if not, allocate a new one, and return its index
-        $i = $self->{graph}->colorAllocate(@_);
+	$self->setAntiAliased($i);
+	eval "$i = gdAntiAliased";
+    }
 
-        # XXX if this fails, we should use colorClosest.
-        # All of this could potentially be done by using colorResolve
-        # The problem is that colorResolve doesn't return an error
-        # condition (-1) if it can't allocate a color. Instead it always
-        # returns 0.
-    } 
     return $i;
 }
 
@@ -1020,6 +1027,15 @@ y_label_skip>.  Default: 1 for both.
 Draw the axes as a box, if true.
 Default: 1.
 
+=item no_axes
+
+Draw no axes at all. If this is set to undef, all axes are drawn. If
+it is set to 0, the zero axis will be drawn, I<for bar charts only>.
+If this is set to a true value, no axes will be drawns at all. Value
+labels on the axes and ticks will also not be drawn, but axis lables
+are drawn.
+Default: undef.
+
 =item two_axes
 
 Use two separate axes for the first and second data set. The first
